      subroutine bstcm( p1, p2,
     >     pcm, e1, e2, bdir, gamma, phi, theta )
C
C  Obtain the parameters to boost the input two-particle system to
C  their center-of-mass frame.
C  The momentum and the energies in the CM frame are returned.
C  The rotation angles to align the momenta along the z-axis are also 
C  obtained, where the 1st particle always directs to the positive side.
C  13 April 2005, S. Odaka (KEK)
C
C  (input)  p1(i): momentum and energy of the 1st particle
C           p2(i): momentum and energy of the 2nd particle
C
C  (output) pcm: momentum of the 1st particle (> 0) in the CM frame
C           e1: energy of the 1st particle in the CM frame
C           e2: energy of the 2nd particle in the CM frame
C           bdir(i): boost direction (unit vector)
C           gamma: gamma value of the Lorentz boost
C           phi: phi rotation angle to make py = 0; px(1) > 0 after
C                the rotation
C           theta: theta rotation angle to make px = 0; pz(1) > 0
C                  after the rotation
C
      implicit real*8(a-h,o-z)
      dimension p1(4),p2(4), bdir(3)
      dimension ptemp(3)

CCC      print *,'BSTCM'

C  Reset the outputs.
      pcm= 0.d0
      e1= 0.d0
      e2= 0.d0
      bdir(1)= 0.d0
      bdir(2)= 0.d0
      bdir(3)= 1.d0
      gamma= 1.d0
      phi= 0.d0
      theta= 0.d0

C  Lorentz-boost parameters.
      esys= p1(4) +p2(4)
      if( esys.lt.0.d0 ) then
         print *,'BSTCM: negative total energy.'
         print '(5X,''p1: '',4E12.4)',(p1(i),i=1,4)
         print '(5X,''p2: '',4E12.4)',(p2(i),i=1,4)
         stop
      endif
      pabs= 0.d0
      do 10 i=1,3
         ptemp(i)= p1(i) +p2(i)
         pabs= pabs +ptemp(i)**2
 10   continue
      s= esys**2 -pabs
      pabs= sqrt( pabs )
      if( s.lt.0.d0 ) then
         print *,'BSTCM: negative s.'
         stop
      endif
      roots= sqrt( s )
      gamma= esys/roots
      beta= sqrt( 1.d0 -1.d0/gamma**2 )
      if( beta.gt.1.d-10 ) then
         do 20 i=1,3
            bdir(i)= ptemp(i)/pabs
 20      continue
CCC      else
CCC         print '('' BSTCM: very small beta.'',E14.6)', beta
      endif

C  Boost the 1st particle to obtain the energy-momentum and rotation 
C  angles.
C  Lorentz boost.
      plin= p1(1)*bdir(1) +p1(2)*bdir(2) +p1(3)*bdir(3)
      do 30 i=1,3
         ptemp(i)= p1(i) -plin*bdir(i)
 30   continue
      ein1= p1(4)
      e1= gamma*( ein1 -beta*plin )
      pl= gamma*( -beta*ein1 +plin )
      do 40 i=1,3
         ptemp(i)= ptemp(i) +pl*bdir(i)
 40   continue
      pcm= sqrt( ptemp(1)**2 +ptemp(2)**2 +ptemp(3)**2 )
CCC      print '(5X,''p(cm) = '',E14.6)', pcm
C  Rotation.
      phi= atan2( ptemp(2), ptemp(1) )
      px= ptemp(1)*cos( phi ) +ptemp(2)*sin( phi )
      theta= atan2( px, ptemp(3) )
CCC      pz= ptemp(3)*cos( theta ) +px*sin( theta )
CCC      print '(5X,''pz (final) = '',E14.6)', pz

C  2nd particle for test.
CCC      plin= p2(1)*bdir(1) +p2(2)*bdir(2) +p2(3)*bdir(3)
CCC      ein2= p2(4)
CCC      e2= gamma*( ein2 -beta*plin )
CCC      print '(5X,''E2 from boost = '',E14.6)', e2

C  The energy of the 2nd particle from root(s).
      e2= roots -e1
CCC      print '(5X,''E2 from root(s) = '',E14.6)', e2

      return
      end

*----+---1---------2---------3---------4---------5---------6---------7-+
      subroutine bstbk( pin, bdir, gamma, phi, theta, pbk )
C
C  Boost back a momentum to the laboratory frame using the rotation and 
C  Lorentz boost parameters determined by the subroutine BSTCM.
C  13 April 2005, S. Odaka (KEK)
C
C  (input)  pin(i): input momentum and energy of the particle
C           bdir(i): direction of the boost (unit vector)
C           gamma: gamma value of the Lorentz boost
C           phi: phi rotation angle
C           theta: theta rotation angle
C
C  (output) pbk(i): momentum and energy boosted back
C
      implicit real*8(a-h,o-z)
      dimension pin(4),pbk(4),bdir(3)

C  Theta rotation.
      sn= sin( theta )
      cs= cos( theta )
      px= pin(1)*cs +pin(3)*sn
      pz= -pin(1)*sn +pin(3)*cs
      py= pin(2)

C  Phi rotation.
      sn= sin( phi )
      cs= cos( phi )
      pbk(1)= px*cs -py*sn
      pbk(2)= px*sn +py*cs
      pbk(3)= pz

C  Lorentz boost.
      beta= sqrt( 1.d0 -1.d0/gamma**2 )
      plin= pbk(1)*bdir(1) +pbk(2)*bdir(2) +pbk(3)*bdir(3)
      do 10 i=1,3
         pbk(i)= pbk(i) -plin*bdir(i)
 10   continue
      plbk= gamma*( beta*pin(4) +plin )
      do 20 i=1,3
         pbk(i)= pbk(i) +plbk*bdir(i)
 20   continue
      pbk(4)= gamma*( pin(4) +beta*plin )

      return
      end

*----+---1---------2---------3---------4---------5---------6---------7-+
      subroutine bstfw( pin, bdir, gamma, phi, theta, pfw )
C
C  Boost the input momentum to the CM frame using the rotation and 
C  Lorentz boost parameters determined by the subroutine BSTCM.
C  25 August 2005, S. Odaka (KEK)
C
C  (input)  pin(i): input momentum and energy of the particle
C           bdir(i): direction of the boost (unit vector)
C           gamma: gamma value of the Lorentz boost
C           phi: phi rotation angle
C           theta: theta rotation angle
C
C  (output) pfw(i): momentum and energy in the CM frame
C
      implicit real*8(a-h,o-z)
      dimension pin(4),pfw(4),bdir(3)

C  Lorentz boost.
      beta= sqrt( 1.d0 -1.d0/gamma**2 )
      plin= pin(1)*bdir(1) +pin(2)*bdir(2) +pin(3)*bdir(3)
      do 10 i=1,3
         pfw(i)= pin(i) -plin*bdir(i)
 10   continue
      plfw= gamma*( -beta*pin(4) +plin )
      do 20 i=1,3
         pfw(i)= pfw(i) +plfw*bdir(i)
 20   continue
      pfw(4)= gamma*( pin(4) -beta*plin )

C  Phi rotation
      px= pfw(1)
      py= pfw(2)
      csph= cos( phi )
      snph= sin( phi )
      pfw(1)= px*csph +py*snph
      pfw(2)= -px*snph +py*csph

C  Theta rotation
      px= pfw(1)
      pz= pfw(3)
      csth= cos( theta )
      snth= sin( theta )
      pfw(1)= px*csth -pz*snth
      pfw(3)= px*snth +pz*csth

      return
      end
